<?php

namespace Drupal\standwithukraine\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Element\CompositeFormElementTrait;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Url;
use Drupal\standwithukraine\StandWithUkraineSettingsInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure StandWithUkraine settings for this site.
 */
class StandWithUkraineForm extends ConfigFormBase implements TrustedCallbackInterface {

  use CompositeFormElementTrait;

  /**
   * The suggested websites list.
   */
  const URLS = [
    'https://supportukrainenow.org',
    'https://war.ukraine.ua/',
  ];

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);

    $instance->renderer = $container->get('renderer');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['standwithukraine.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'standwithukraine_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);

    $description = $this->t('Take into account the following rules:');

    $element = [
      '#theme' => 'item_list',
      '#items' => [
        $this->t('Empty lines will be deleted and other lines will be combined
when their number is more than two.'),
        $this->t(
          'When a field is empty then a message will contain the following
phrase - %text.',
          [
            '%text' => str_replace(
              PHP_EOL,
              ' ',
              StandWithUkraineSettingsInterface::TEXT
            ),
          ]
        ),
      ],
    ];

    $description .= $this->renderer->render($element);

    $form['text'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Text'),
      '#description' => $description,
      '#rows' => 2,
      '#resizable' => 'horizontal',
      '#default_value' => $config->get('text'),
    ];

    $form['single'] = [
      '#type' => 'radios',
      '#title' => $this->t('Lines number'),
      '#description' => $this->t('Define if the message\'s text should be split
or displayed as a single row.'),
      '#options' => [
        1 => $this->t('One'),
        0 => $this->t('Two'),
      ],
      '#default_value' => (int) $config->get('single'),
    ];

    $form['sizes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Bigger font size'),
      '#description' => $this->t('This may be necessary if you want to increase
the text size of one of the lines to bring its width closer to the width of the
other.'),
      '#options' => [
        'first' => $this->t('First line'),
        'last' => $this->t('Second line'),
      ],
      '#default_value' => array_keys(array_filter((array) $config->get('sizes'))),
      // Since the code checker can not understand this logic yet so skip
      // checking it.
      // @noRector
      // @phpstan-ignore-next-line
      '#pre_render' => array_map(function ($callback) {
        return [self::class, $callback];
      }, self::trustedCallbacks()),
    ];

    $form['url'] = [
      '#type' => 'url',
      '#title' => $this->t('Link'),
      '#description' => sprintf(
        '%s %s.',
        $this->t('For example:'),
        implode(
          ', ',
          array_map(
            function ($url) {
              return Link::fromTextAndUrl(
                parse_url($url, PHP_URL_HOST),
                Url::fromUri($url, ['target' => '_blank'])
              )->toString();
            },
            self::URLS
          )
        )
      ),
      '#default_value' => $config->get('url'),
    ];

    $form['position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Position'),
      '#description' => $this->t('Select on which side of the page a message
should be displayed.'),
      '#options' => [
        StandWithUkraineSettingsInterface::POSITION_LEFT => $this->t('Left'),
        StandWithUkraineSettingsInterface::POSITION_TOP => $this->t('Top (default)'),
        StandWithUkraineSettingsInterface::POSITION_RIGHT => $this->t('Right'),
      ],
      '#default_value' => $config->get('position') ?? '',
    ];

    $form['offset'] = [
      '#type' => 'number',
      '#title' => $this->t('Offset'),
      '#description' => $this->t('How many pixels down the message should be
shifted down the page? When a field is empty then the message will be shifted
down by 20 percent from the page height. Available only for the left or right
position of the message.'),
      '#min' => 0,
      '#field_suffix' => $this->t('pixels'),
      '#default_value' => $config->get('offset'),
      '#states' => [
        'enabled' => [
          ':input[name="position"]' => [
            ['value' => 'left'],
            ['value' => 'right'],
          ],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config($this->getEditableConfigNames()[0]);
    $values = $form_state->getValues();

    $lines = &$values['text'];
    $lines = explode(PHP_EOL, $lines);

    array_walk($lines, function (&$line) {
      $line = trim($line);
    });

    if (($count = count($lines)) > 2) {
      $lines = [
        array_slice($lines, 0, $count = round($count / 2)),
        array_slice($lines, $count),
      ];

      array_walk($lines, function (&$line) {
        $line = implode(' ', $line);
      });
    }

    $lines = implode(PHP_EOL, array_filter($lines));

    foreach (['text', 'single', 'sizes', 'url', 'position', 'offset'] as $key) {
      $config->set($key, $values[$key]);
    }

    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Describe when font size can be changed for specific lines.
   *
   * Also, disable the font size option of the second line when a single-line
   * view is used.
   *
   * @param array $element
   *   An associative array containing the properties and children of the
   *   element.
   *
   * @return array
   *   The changed form element render array.
   */
  public static function preRenderSizes(array $element) {
    $element['first']['#description'] = t('Available when a message is displayed
in the single-line view or double-line view.');

    $element['last']['#description'] = t('Available only when a message is
displayed in the double-line view.');

    $element['last']['#states'] = [
      'disabled' => [
        ':input[name="single"]' => [
          'value' => '1',
        ],
      ],
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRenderCompositeFormElement', 'preRenderSizes'];
  }

}
